
#include "PurebredData.h"
#include "Mouse.h"

#include <vector>
#include <string>

using namespace std;

// ================================================================
// GenomeMixer    (c) 2003, Alex Williams
// All rights reserved.
// Send email correspondence to Robert W. Williams <rwilliam@nb.utmem.edu>
// at the University of Tennesse Health Science Center

// This project was funded jointly by the National Institute of Mental Health, National Institute on Drug Abuse,
// and the National Science Foundation (award P20-MH 62009 to KFM and RW)

// This source code may be used according to the terms of the GNU General Public License (GPL).
// See www.gnu.org for specifics. Essentially, the source may be freely used as long as the source code
// for whatever program the source is also made freely available. If you want to use this source
// in a closed-source program, you should email us for licensing options.

// Disclaimer: This code comes with no warranty at all, and there is no guarantee that it will
// work as claimed in the documentation. Hopefully, it will be useful anyway.

// Note that GenomeMixer makes use of the Trolltech Qt toolkit for its window-managing needs.
// There is very little use of Qt data structures (aside from QString) in the actual processing code,
// but **you will need to have Qt installed in order to compile this code.** Check <www.trolltech.com>
// for details; there should be a free GPL-software-only version of Qt available on the Trolltech
// web site.

// Any redistributions of this software in code or binary form must include the
// above copyright notice, licensing information, disclaimer, and this notice itself.
// Basically, all the comments at the top of this file must be retained.
// ================================================================


// ==============================================================

std::string* PurebredData::getPureSymbolPtr(int index) {
	if (index < 0 || index >= numPurebreds()) {
		cout << "PurebredData:getPureNamePtr: Failing to get purebred symbol at this index: " << index << endl;
		throw new string("We attempted to get a purebred symbol at a nonexistent index.\nThis should not happen, but it can happen if you\ndelete some of the parental strains\nafter using them elsewhere.");
	}
	return &(parentalSymbols[index]); // return a pointer to the name (a std:string)
}

string* PurebredData::getPureNamePtr(int index) {
	if (index < 0 || index >= numPurebreds()) {
		cout << "PurebredData:getPureNamePtr: Failing to get purebred name at this index: " << index << endl;
		throw new string("We attempted to get a purebred name at a nonexistent index.\nThis should not happen, but it can happen if you\ndelete some of the parental strains\nafter using them elsewhere.");
	}
	return &(parentalStrains[index]); // return a pointer to the name (a std:string)
}

Mouse* PurebredData::getPurePtr(int index, bool sex) {
	// Returns a pointer to a purebred parental strain mouse
	if (sex == kMALE) {
		if (index < 0 || index >= long(purebredMaleVec.size())) {
			cout << "PurebredData:getPureNamePtr: Failing to get purebred name at this index: " << index << endl;
			throw new string("We attempted to get a purebred individual (male)\nat a nonexistent index.\nAlthough this is a mistake, and shouldn't happen, it can happen if you\ndelete some of the parental strains\nafter using them elsewhere.");
		}
		return purebredMaleVec[index];
	} else {
		if (index < 0 || index >= long(purebredFemaleVec.size())) {
			cout << "PurebredData:getPureNamePtr: Failing to get purebred name at this index: " << index << endl;
			throw new string("We attempted to get a purebred individual (female)\nat a nonexistent index.\nAlthough this is a mistake, and shouldn't happen, it can happen if you\ndelete some of the parental strains\nafter using them elsewhere.");
		}
		return purebredFemaleVec[index];
	}
}


void PurebredData::clearAll(void) {
	// Erases all the ancestral strain (purebred) information
	Mouse::resetID_Counter();; // <-- have to reset the counter, or everything becomes weird
	parentalStrains.reserve(0);
	parentalStrains.clear();
	parentalSymbols.reserve(0);
	parentalSymbols.clear();
}

void PurebredData::addPurebred(string strainName, string symbol) { 
	// Adds a (purebred) parental strain
	parentalStrains.push_back(strainName);
	parentalSymbols.push_back(symbol);
}


int PurebredData::numPurebreds() {
	return parentalStrains.size(); // <-- accessor
}


void PurebredData::makePurebredMice(void) {
	purebredMaleVec.reserve(0);
	purebredMaleVec.clear();
	
	purebredFemaleVec.reserve(0);
	purebredFemaleVec.clear();
	// make purebred mice from the parentalStrains array (AA, BB, etc)
	for (int i = 0; i < numPurebreds(); i++) {
		purebredMaleVec.push_back(  Mouse::returnNewPurebredPtr(i, *(this->getPureNamePtr(i)), kMALE) );
		purebredFemaleVec.push_back(Mouse::returnNewPurebredPtr(i, *(this->getPureNamePtr(i)), kFEMALE) );
		//# make purebreds from the breeding scheme file
	}
}
