/* The main program for GenomeMixer. Alex Williams */

#include <iostream>
#include <stdlib.h>
#include <time.h>

#include "MouseIO.h"
#include "Chromosome.h"
#include "GenomeMixer.h"

using namespace std;

// •••••••• •••••••• •••••••• •••••••• •••••••• •••••••• ••••••••
//				  MAIN	PROGRAM
// •••••••• •••••••• •••••••• •••••••• •••••••• •••••••• ••••••••
int mainCommandLine (int argc, const char * argv[]) {
    
    time_t  startTime; // <-- This gives time in seconds (time_t is a long int, usually, also)
    time(&startTime); // "startTime = current time"
    srand(startTime); // initialize the random seed
    
    GenomeMixer *mainMixer = new GenomeMixer();
    
    cout << "-------------------------------------------------------------------\n";
    cout << "@@ Running Genome Mixer, A C++ program by Alex Williams. Version 0.52c, July 2003." << endl;
    cout << "@@ See the documentation online at http://www.nervenet.org/genome_mixer/" << endl;
    cout << "@@ Here are the current settings (currently compiled into the program--bad!):" << endl;
    cout << "@@   > Full path start: \"" << mainMixer->fullPathStart << "\"" << endl;
    cout << "@@   > Name of config file: \"" << mainMixer->configFilename << "\"" << endl;
    cout << "@@   > Name of marker file: \"" << mainMixer->markerLocationsFilename << "\"" << endl;
    cout << "@@   > Path to find these files: \"" << mainMixer->fullPathStart << "\" (an empty path is OK with Mac OS 9, but NOT for other platforms)" << endl;
    cout << "@@ Types of files to output:" << endl;
    
    if (mainMixer->outputQGeneFiles) cout << "     > QGene format: YES" << endl;
    else                             cout << "     > QGene format: NO" << endl;
    
    if (mainMixer->outputMManagerQTXFiles) cout << "     > MapManager QTX format: YES" << endl;
    else                                   cout << "     > MapManager QTX format: NO" << endl;
    
    cout << "@@ Minimum distance between recombinations: " << mainMixer->noRecombDistBases << " bases (absolute threshold)" << endl;
    
    if (!mainMixer->outputQGeneFiles && !mainMixer->outputMManagerQTXFiles) {
	cout << ">>> Warning: Both QGene output and MapManager output were set to \"false\".\n";
	cout << ">>> This will result in calculations being done, but the results NOT being output.\n";
	cout << ">>> You will want to set one of the outputQGeneFiles and outputMManagerQTXFiles variables\n";
	cout << "    to \"true.\" See the readme online for more details.\n";
	cout << "@@ No files output.\n";
    }
    
    // Next line: read the breeding scheme file
    MouseIO::readBreedingSchemeFile((mainMixer->fullPathStart + mainMixer->configFilename), mainMixer);

    // Next line: read the marker location file
    mainMixer->markerColPtr = MouseIO::getMarkerDataFromFile(mainMixer->fullPathStart + mainMixer->markerLocationsFilename);
    //mainMixer->markerColPtr->printMarkerInfo(); // <-- For debugging, prints out the marker data the we just read in
        
    mainMixer->makePurebreds();	// <-- make the purebred strains
    mainMixer->runSimulation(); // <-- this is where all of the actual processing takes place (in GenomeMixer.cpp)
    
    // *************** PRINTING DEBUGGING DATA HERE **************

    //Chromosome::printClassData(); // prints out all the chromosome data we just read in
    //mainMixer->printBreedingTable();    //  <-- for debugging: outputs the breeding scheme table (should be the same as in the file specified by $configFilename)
    //mainMixer->printParentalStrains();
    mainMixer->printListOfExportMice();     //  <-- for debugging: prints the list of mice that are to be exported
    
    // Only prints half the mices
    //for (int i = 0; i < mainMixer->numMiceToExport(); i+= 2) {
	//mainMixer->mouseToExport(i)->printMouseData();
    //}
    
    // *************** WRITING OUTPUT FILES HERE **************
    if (mainMixer->outputQGeneFiles) {
	//MouseIO::writeBreedingSchemeFile("/GenomeMixerCPP/randomOutput.txt");
	//MouseIO::writeQGeneFiles($UserDefined::fullPathStart . $UserDefined::QGeneOutputFilename, \$Glob::mice, \$markerData);
	cout << ">> Note (from qgene.org): \"A dataset may have as many as 2048 markers, 2048 individuals, and 500 traits.\"\n";
	cout << ">> Note: QGene output is currently only compatible with crosses that originate from AA and BB parental strains. If you have a CC parental strain, the QGene output will not work properly.\n";
	cout << "@@ Wrote a QGene population file name \"$UserDefined::QGeneOutputFilename.pop\"\n";
		cout << "@@         and a QGene map file named \"$UserDefined::QGeneOutputFilename.map\".\n";
    }
    if (mainMixer->outputMManagerQTXFiles) {
	//MouseIO::writeMapManagerFormat($UserDefined::fullPathStart . $UserDefined::MManagerQTXFilename, \$Glob::mice, \$markerData);
	cout << "@@ Wrote a MapManager QTX format file named \"$UserDefined::MManagerQTXFilename\".\n";
    }
    // *************** FINISHED WRITING OUTPUT FILES **************
    
    cout << "@@ (" << (time(NULL) - startTime) << " seconds elapsed)" << endl;
    cout << "-------------------------------------------------------------------" << endl;
    
    return 0;
}






/*
http://turing.une.edu.au/~comp132/FAQs/vectorRef.html

C++ vector Library Quick Reference

vector Function
Description

vector aVec<Type>;
Declare aVec as an empty vector              for storing Type values

vector aVec<Type>(n);
Declare aVec as a vector              containing n default values of type              Type

vector aVec<Type>(n, val);
Declare aVec as a vector              containing n copies of value val 

aVec[i]
Access the character in aVec whose index is i             (without checking that i is valid)

aVec.at(i)
Access the character in aVec whose index is i             (checking that i is valid)

aVec.size()
Return the number of values in aVec

aVec.capacity()
Return the number of values aVec can store

aVec.reserve(n)
Change the capacity of aVec to n

aVec.push_back(value);
Append value at aVec's end

aVec.pop_back();
Erase the last value in aVec

aVec.front()
Return a reference to the first element of aVec

aVec.back()
Return a reference to the last element of aVec

aVec = vec2
Make aVec a copy of vec2

aVec.clear()
Erase all of the values in aVec

         vector Functions Using iterators     
aVec.begin()
Return an iterator to the first element of aVec

aVec.end()
Return an iterator pointing beyond the last element of aVec

aVec.erase(it)
Remove the element of aVec pointed to by iterator it

aVec.insert(it, val)
Insert value val before the element of aVec pointed to by iterator it

    

          Copyright 1998 by         Joel C. Adams.        All rights reserved.       
 Modified 2002 to suit COMP132 at UNE.         

*/

